//-------------------------------------------------------------------
// code03e9.c
//-------------------------------------------------------------------

#include <PalmOs.h>
#include <HostControl.h>

#ifdef _SONY
# include <SonyChars.h>
#else
# include <TrgChars.h>
#endif

#include "ELaunch.h"
#include "Resource.h"

/*##################################################################*/
asm ( " bra MySysHandleEvent" );
/*##################################################################*/

typedef struct
{
  TPrefs	    Prefs;
  UInt16	    chr;
  Boolean	    PrefsOK;
  UInt32	    KeyPrimBit;
  int		    KeyPrim;
  UInt16	    wSize;
  UInt16	    initDelay;
  UInt16	    period;
  UInt16	    doubleTapDelay;
  Boolean	    queueAhead;
  UInt32	    HoldTime;
  UInt32	    HoldTimeLong;
  TGlobals	    *pVars;
  Boolean	    PrimKeyReleased;
  UInt32	    SecKeysBit;
  UInt32	    HoldKeysBit;
  UInt32	    CurKeys;
  int		    KeyStyle;
  int		    KeySec;
  int		    TryKeyStyle;
  int		    TryKeySec;
  UInt32	    Ticks;
  EventType	    Event;
  TCommonVars	    CommonVars;
} TVars;

/*##################################################################*/
Boolean MySysHandleEvent ( EventPtr pEvent )
/*##################################################################*/
{
  Boolean Handled = false;
  Boolean (*oldtrap)( EventPtr );	

  if ( pEvent->eType == keyDownEvent )
  {
    TVars *vars = (TVars*)MemPtrNew( sizeof( *vars ) );

#ifdef TRACKME
    {
      const static char gszFormat[] = "SysHandleEvent chr:%#08hx mod:%#08hx code:%#08hx";

      HostTraceInit();
      HostTraceOutputTL(appErrorClass, gszFormat, 
	pEvent->data.keyDown.chr, pEvent->data.keyDown.modifiers, pEvent->data.keyDown.keyCode );
    }
#endif

    if ( NULL != vars )
    {
      vars->KeyPrimBit = 0;
      vars->chr = pEvent->data.keyDown.chr;
      vars->wSize = sizeof( vars->Prefs );

      vars->PrefsOK = APPPREFVER == PrefGetAppPreferences( HACKID, APPPREFID, &(vars->Prefs), &(vars->wSize), true );

      if ( vars->PrefsOK && vars->Prefs.CheckBoxes[ cClickOnGraffiti ] && 0 == ( pEvent->data.keyDown.modifiers & commandKeyMask ) )
        SndPlaySystemSound( sndClick );

      if ( vars->chr >= hard1Chr && vars->chr <= hard4Chr )
      {
        vars->KeyPrimBit = keyBitHard1 << ( vars->chr - hard1Chr );
        vars->KeyPrim = pDATEBOOK - hard1Chr + vars->chr;
      }
      else if ( vars->chr == hardPowerChr )
      {
        vars->KeyPrimBit = keyBitPower;
        vars->KeyPrim = pPOWER;
      }
      else if ( vars->chr == hardCradleChr )
      {
        vars->KeyPrimBit = keyBitCradle;
        vars->KeyPrim = pCRADLE;
      }
      else if ( vars->chr == hardContrastChr )
      {
        vars->KeyPrimBit = keyBitContrast;
        vars->KeyPrim = pCONTRAST;
      }

#ifdef _SONY
      else if ( vars->chr == vchrJogUp )
      {
        vars->KeyPrim = pJOGUP;
        vars->KeyPrimBit = 0x1000;
      }
      else if ( vars->chr == vchrJogDown )
      {
        vars->KeyPrim = pJOGDOWN;
        vars->KeyPrimBit = 0x1000;
      }
      else if ( vars->chr == vchrJogPush )
      {
        vars->KeyPrim = pJOGPUSH;
        vars->KeyPrimBit = 0x1000;
      }
      else if ( vars->chr == vchrJogBack )
      {
        vars->KeyPrim = pAUX;
        vars->KeyPrimBit = 0x1000;
      }
#else
      else if ( vars->chr >= vchrTrgJogUp && vars->chr <= vchrTrgAux )
      {
        vars->KeyPrimBit = keyBitJogUp << ( vars->chr - vchrTrgJogUp );
        vars->KeyPrim = pJOGUP - vchrTrgJogUp + vars->chr;
      }
#endif

      if ( vars->KeyPrimBit )
      {
        KeyRates( false, &(vars->initDelay), &(vars->period), &(vars->doubleTapDelay), &(vars->queueAhead) );
        vars->HoldTime = TimGetTicks() + vars->initDelay;
        vars->HoldTimeLong = vars->HoldTime + vars->initDelay;

        vars->pVars = NULL;
        FtrGet( HACKID, FTR_VARS, (UInt32*)&(vars->pVars) );

        if ( vars->PrefsOK && NULL == vars->pVars )
        {
          for ( vars->TryKeyStyle = sKEYSTYLECOUNT ; --(vars->TryKeyStyle) >= 0 ; )
            for ( vars->TryKeySec = KEYSECCOUNT ; --(vars->TryKeySec) >= 0 ; )
              if ( 0 != vars->Prefs.aKeys[ KeyIndex( vars->KeyPrim, vars->TryKeySec, vars->TryKeyStyle ) ] )
        	goto PRIMKEYNOTEMPTY;

          goto FALLTHROUGH;

          PRIMKEYNOTEMPTY:;
        }

        for ( vars->KeyStyle = sPRESS , vars->SecKeysBit = 0 , vars->PrimKeyReleased = false ; 0 != ( vars->CurKeys = KeyCurrentState() ) ; )
        {
          if ( !( vars->CurKeys & vars->KeyPrimBit ) )
            vars->PrimKeyReleased = true;

          if ( !vars->PrimKeyReleased )
          {
	    vars->Ticks = TimGetTicks();

	    if ( vars->Ticks > vars->HoldTimeLong )
	    {
	      if ( vars->KeyStyle != sHOLDLONG )
	      {
	        vars->KeyStyle = sHOLDLONG;
	        vars->HoldKeysBit = vars->CurKeys;
	        if ( vars->Prefs.CheckBoxes[ cClickOnHold ] )
	          SndPlaySystemSound( sndClick );
	      }
	    }
	    else
	      if ( vars->Ticks > vars->HoldTime )
	      {
	        if ( vars->KeyStyle != sHOLD )
	        {
	          vars->KeyStyle = sHOLD;
	          vars->HoldKeysBit = vars->CurKeys;
	    	  if ( vars->Prefs.CheckBoxes[ cClickOnHold ] )
        	    SndPlaySystemSound( sndClick );
	        }
	      }
	      else
	        vars->SecKeysBit |= vars->CurKeys;
          }
        }

        if ( vars->KeyStyle > sPRESS )
        {
          vars->SecKeysBit = vars->HoldKeysBit;
          EvtFlushKeyQueue();
        }

        vars->SecKeysBit &= ~( vars->KeyPrimBit );

        if ( 0 != vars->SecKeysBit )
          EvtFlushKeyQueue();

        for ( vars->KeySec = KEYSECCOUNT ; --(vars->KeySec) >= 1 ; )
          if ( vars->SecKeysBit == ( (UInt32)1 << ( vars->KeySec - 1 ) ) )
            break;

        if ( NULL != vars->pVars )
        {
          vars->Event.eType = EVENT_KEY;
          vars->Event.data.generic.datum[0] = vars->KeyPrim;
          vars->Event.data.generic.datum[1] = vars->KeySec;
          vars->Event.data.generic.datum[2] = vars->KeyStyle;
          EvtAddEventToQueue( &(vars->Event) );
          Handled = true;
        }
        else
        {
          if ( ( vars->PrefsOK && DoAnyActionProxy( vars->Prefs.aKeys[ KeyIndex( vars->KeyPrim, vars->KeySec, vars->KeyStyle ) ], &(vars->Prefs), &(vars->CommonVars) ) ) ||
	       vars->KeyStyle > sPRESS )
            Handled = true;
        }

      FALLTHROUGH:
      }

      MemPtrFree( vars );
    }
  }


  if ( Handled )
    return true;
  else
  {
    FtrGet( HACKID, HACKINDEX2, (UInt32*)&oldtrap );
    return oldtrap( pEvent );
  }
}
/*##################################################################*/
#include "common.c"
/*##################################################################*/
